@@ -22,8 +22,10 @@
 #ifndef AWESOME_AWESOME_H
 #define AWESOME_AWESOME_H
 
+#include <stdbool.h>
+
 void awesome_restart(void);
-void awesome_atexit(void);
+void awesome_atexit(bool restart);
 
 #endif
 // vim: filetype=c:expandtab:shiftwidth=4:tabstop=8:softtabstop=4:encoding=utf-8:textwidth=80
