@@ -1038,7 +1038,7 @@ client_set_fullscreen(lua_State *L, int cidx, bool s)
         {
             /* The titlebar was banned, unban! */
             titlebar_unban(c->titlebar);
-            geometry = c->geometries.fullscreen;
+            geometry = titlebar_geometry_add(c->titlebar, 0, c->geometries.fullscreen);
             c->fullscreen = false;
             client_set_border_width(L, cidx, c->border_width_fs);
         }
@@ -1551,7 +1551,26 @@ luaA_client_tags(lua_State *L)
     {
         luaA_checktable(L, 2);
         for(int i = 0; i < tags->len; i++)
-            untag_client(c, tags->tab[i]);
+        {
+            /* Only untag if we aren't going to add this tag again */
+            bool found = false;
+            lua_pushnil(L);
+            while(lua_next(L, 2))
+            {
+                tag_t *t = lua_touserdata(L, -1);
+                /* Pop the value from lua_next */
+                lua_pop(L, 1);
+                if (t != tags->tab[i])
+                    continue;
+
+                /* Pop the key from lua_next */
+                lua_pop(L, 1);
+                found = true;
+                break;
+            }
+            if(!found)
+                untag_client(c, tags->tab[i]);
+        }
         lua_pushnil(L);
         while(lua_next(L, 2))
             tag_client(c);
