@@ -22,6 +22,7 @@
 #ifndef AWESOME_EVENT_H
 #define AWESOME_EVENT_H
 
+#include "systray.h"
 #include "wibox.h"
 #include "client.h"
 
@@ -30,11 +31,12 @@ awesome_refresh(void)
 {
     banning_refresh();
     wibox_refresh();
+    systray_refresh();
     client_stack_refresh();
     return xcb_flush(globalconf.connection);
 }
 
-void a_xcb_set_event_handlers(void);
+void event_handle(xcb_generic_event_t *event);
 
 #endif
 // vim: filetype=c:expandtab:shiftwidth=4:tabstop=8:softtabstop=4:encoding=utf-8:textwidth=80
