@@ -53,10 +53,6 @@ typedef struct
 {
     /** Connection ref */
     xcb_connection_t *connection;
-    /** Event and error handlers */
-    xcb_event_handlers_t evenths;
-    /** Property change handler */
-    xcb_property_handlers_t prophs;
     /** Default screen number */
     int default_screen;
     /** Keys symbol table */
@@ -135,6 +131,8 @@ typedef struct
     wibox_array_t wiboxes;
     /** The startup notification display struct */
     SnDisplay *sndisplay;
+    /** Latest timestamp we got from the X server */
+    xcb_timestamp_t timestamp;
 } awesome_t;
 
 extern awesome_t globalconf;
