@@ -39,11 +39,13 @@
         bool no_reply = !reply; \
         if(no_reply) \
             reply = xcb_get_property_reply(globalconf.connection, \
-                                           xcb_get_any_property(globalconf.connection, \
-                                                                false, \
-                                                                c->window, \
-                                                                atom, \
-                                                                UINT_MAX), NULL); \
+                                           xcb_get_property(globalconf.connection, \
+                                                            false, \
+                                                            c->window, \
+                                                            atom, \
+                                                            XCB_GET_PROPERTY_TYPE_ANY, \
+                                                            0, \
+                                                            UINT_MAX), NULL); \
         luaA_object_push(globalconf.L, c); \
         setfunc(globalconf.L, -1, xutil_get_text_property_from_reply(reply)); \
         lua_pop(globalconf.L, 1); \
@@ -51,9 +53,7 @@
             p_delete(&reply); \
     } \
     static int \
-    property_handle_##funcname(void *data, \
-                               xcb_connection_t *connection, \
-                               uint8_t state, \
+    property_handle_##funcname(uint8_t state, \
                                xcb_window_t window, \
                                xcb_atom_t name, \
                                xcb_get_property_reply_t *reply) \
@@ -65,20 +65,18 @@
     }
 
 
-HANDLE_TEXT_PROPERTY(wm_name, WM_NAME, client_set_alt_name)
+HANDLE_TEXT_PROPERTY(wm_name, XCB_ATOM_WM_NAME, client_set_alt_name)
 HANDLE_TEXT_PROPERTY(net_wm_name, _NET_WM_NAME, client_set_name)
-HANDLE_TEXT_PROPERTY(wm_icon_name, WM_ICON_NAME, client_set_alt_icon_name)
+HANDLE_TEXT_PROPERTY(wm_icon_name, XCB_ATOM_WM_ICON_NAME, client_set_alt_icon_name)
 HANDLE_TEXT_PROPERTY(net_wm_icon_name, _NET_WM_ICON_NAME, client_set_icon_name)
-HANDLE_TEXT_PROPERTY(wm_client_machine, WM_CLIENT_MACHINE, client_set_machine)
+HANDLE_TEXT_PROPERTY(wm_client_machine, XCB_ATOM_WM_CLIENT_MACHINE, client_set_machine)
 HANDLE_TEXT_PROPERTY(wm_window_role, WM_WINDOW_ROLE, client_set_role)
 
 #undef HANDLE_TEXT_PROPERTY
 
 #define HANDLE_PROPERTY(name) \
     static int \
-    property_handle_##name(void *data, \
-                           xcb_connection_t *connection, \
-                           uint8_t state, \
+    property_handle_##name(uint8_t state, \
                            xcb_window_t window, \
                            xcb_atom_t name, \
                            xcb_get_property_reply_t *reply) \
@@ -140,7 +138,7 @@ property_update_wm_client_leader(client_t *c, xcb_get_property_reply_t *reply)
     if(no_reply)
     {
         client_leader_q = xcb_get_property_unchecked(globalconf.connection, false, c->window,
-                                                     WM_CLIENT_LEADER, WINDOW, 0, 32);
+                                                     WM_CLIENT_LEADER, XCB_ATOM_WINDOW, 1, 32);
 
         reply = xcb_get_property_reply(globalconf.connection, client_leader_q, NULL);
     }
@@ -244,9 +242,7 @@ property_update_wm_class(client_t *c, xcb_get_property_reply_t *reply)
 }
 
 static int
-property_handle_net_wm_strut_partial(void *data,
-                                     xcb_connection_t *connection,
-                                     uint8_t state,
+property_handle_net_wm_strut_partial(uint8_t state,
                                      xcb_window_t window,
                                      xcb_atom_t name,
                                      xcb_get_property_reply_t *reply)
@@ -286,7 +282,7 @@ property_update_net_wm_pid(client_t *c,
     if(no_reply)
     {
         xcb_get_property_cookie_t prop_c =
-            xcb_get_property_unchecked(globalconf.connection, false, c->window, _NET_WM_PID, CARDINAL, 0L, 1L);
+            xcb_get_property_unchecked(globalconf.connection, false, c->window, _NET_WM_PID, XCB_ATOM_CARDINAL, 0L, 1L);
         reply = xcb_get_property_reply(globalconf.connection, prop_c, NULL);
     }
 
@@ -340,17 +336,13 @@ property_update_wm_protocols(client_t *c, xcb_get_property_reply_t *reply)
 }
 
 /** The property notify event handler.
- * \param data currently unused.
- * \param connection The connection to the X server.
  * \param state currently unused
  * \param window The window to obtain update the property with.
  * \param name The protocol atom, currently unused.
  * \param reply (Optional) An existing reply.
  */
 static int
-property_handle_xembed_info(void *data __attribute__ ((unused)),
-                            xcb_connection_t *connection,
-                            uint8_t state,
+property_handle_xembed_info(uint8_t state,
                             xcb_window_t window,
                             xcb_atom_t name,
                             xcb_get_property_reply_t *reply)
@@ -358,15 +350,13 @@ property_handle_xembed_info(void *data __attribute__ ((unused)),
     xembed_window_t *emwin = xembed_getbywin(&globalconf.embedded, window);
 
     if(emwin)
-        xembed_property_update(connection, emwin, reply);
+        xembed_property_update(globalconf.connection, emwin, reply);
 
     return 0;
 }
 
 static int
-property_handle_xrootpmap_id(void *data __attribute__ ((unused)),
-                             xcb_connection_t *connection,
-                             uint8_t state,
+property_handle_xrootpmap_id(uint8_t state,
                              xcb_window_t window,
                              xcb_atom_t name,
                              xcb_get_property_reply_t *reply)
@@ -376,7 +366,7 @@ property_handle_xrootpmap_id(void *data __attribute__ ((unused)),
             (*w)->need_update = true;
     else
     {
-        int screen = xutil_root2screen(connection, window);
+        int screen = xutil_root2screen(globalconf.connection, window);
         foreach(w, globalconf.wiboxes)
             if(screen == screen_array_indexof(&globalconf.screens, (*w)->screen))
                (*w)->need_update = true;
@@ -386,9 +376,7 @@ property_handle_xrootpmap_id(void *data __attribute__ ((unused)),
 }
 
 static int
-property_handle_net_wm_opacity(void *data __attribute__ ((unused)),
-                               xcb_connection_t *connection,
-                               uint8_t state,
+property_handle_net_wm_opacity(uint8_t state,
                                xcb_window_t window,
                                xcb_atom_t name,
                                xcb_get_property_reply_t *reply)
@@ -415,54 +403,79 @@ property_handle_net_wm_opacity(void *data __attribute__ ((unused)),
     return 0;
 }
 
-void a_xcb_set_property_handlers(void)
+/** The property notify event handler.
+ * \param data Unused data.
+ * \param connection The connection to the X server.
+ * \param ev The event.
+ * \return Status code, 0 if everything's fine.
+ */
+void
+property_handle_propertynotify(xcb_property_notify_event_t *ev)
 {
-    /* init */
-    xcb_property_handlers_init(&globalconf.prophs, &globalconf.evenths);
+    uint32_t length;
+    int (*handler)(uint8_t state,
+                   xcb_window_t window,
+                   xcb_atom_t name,
+                   xcb_get_property_reply_t *reply) = NULL;
+
+    globalconf.timestamp = ev->time;
+
+    /* Find the correct event handler */
+#define HANDLE(atom_, cb, len) \
+    if (ev->atom == atom_) \
+    { \
+        handler = cb; \
+        length = len; \
+    } else
+#define HANDLE_L(atom, cb) HANDLE(atom, cb, UINT_MAX)
+#define HANDLE_S(atom, cb) HANDLE(atom, cb, 1)
+#define END return
 
     /* Xembed stuff */
-    xcb_property_set_handler(&globalconf.prophs, _XEMBED_INFO, UINT_MAX,
-                             property_handle_xembed_info, NULL);
+    HANDLE_L(_XEMBED_INFO, property_handle_xembed_info)
 
     /* ICCCM stuff */
-    xcb_property_set_handler(&globalconf.prophs, WM_TRANSIENT_FOR, UINT_MAX,
-                             property_handle_wm_transient_for, NULL);
-    xcb_property_set_handler(&globalconf.prophs, WM_CLIENT_LEADER, UINT_MAX,
-                             property_handle_wm_client_leader, NULL);
-    xcb_property_set_handler(&globalconf.prophs, WM_NORMAL_HINTS, UINT_MAX,
-                             property_handle_wm_normal_hints, NULL);
-    xcb_property_set_handler(&globalconf.prophs, WM_HINTS, UINT_MAX,
-                             property_handle_wm_hints, NULL);
-    xcb_property_set_handler(&globalconf.prophs, WM_NAME, UINT_MAX,
-                             property_handle_wm_name, NULL);
-    xcb_property_set_handler(&globalconf.prophs, WM_ICON_NAME, UINT_MAX,
-                             property_handle_wm_icon_name, NULL);
-    xcb_property_set_handler(&globalconf.prophs, WM_CLASS, UINT_MAX,
-                             property_handle_wm_class, NULL);
-    xcb_property_set_handler(&globalconf.prophs, WM_PROTOCOLS, UINT_MAX,
-                             property_handle_wm_protocols, NULL);
-    xcb_property_set_handler(&globalconf.prophs, WM_CLIENT_MACHINE, UINT_MAX,
-                             property_handle_wm_client_machine, NULL);
-    xcb_property_set_handler(&globalconf.prophs, WM_WINDOW_ROLE, UINT_MAX,
-                             property_handle_wm_window_role, NULL);
+    HANDLE_L(XCB_ATOM_WM_TRANSIENT_FOR, property_handle_wm_transient_for)
+    HANDLE_L(WM_CLIENT_LEADER, property_handle_wm_client_leader)
+    HANDLE_L(XCB_ATOM_WM_NORMAL_HINTS, property_handle_wm_normal_hints)
+    HANDLE_L(XCB_ATOM_WM_HINTS, property_handle_wm_hints)
+    HANDLE_L(XCB_ATOM_WM_NAME, property_handle_wm_name)
+    HANDLE_L(XCB_ATOM_WM_ICON_NAME, property_handle_wm_icon_name)
+    HANDLE_L(XCB_ATOM_WM_CLASS, property_handle_wm_class)
+    HANDLE_L(WM_PROTOCOLS, property_handle_wm_protocols)
+    HANDLE_L(XCB_ATOM_WM_CLIENT_MACHINE, property_handle_wm_client_machine)
+    HANDLE_L(WM_WINDOW_ROLE, property_handle_wm_window_role)
 
     /* EWMH stuff */
-    xcb_property_set_handler(&globalconf.prophs, _NET_WM_NAME, UINT_MAX,
-                             property_handle_net_wm_name, NULL);
-    xcb_property_set_handler(&globalconf.prophs, _NET_WM_ICON_NAME, UINT_MAX,
-                             property_handle_net_wm_icon_name, NULL);
-    xcb_property_set_handler(&globalconf.prophs, _NET_WM_STRUT_PARTIAL, UINT_MAX,
-                             property_handle_net_wm_strut_partial, NULL);
-    xcb_property_set_handler(&globalconf.prophs, _NET_WM_ICON, UINT_MAX,
-                             property_handle_net_wm_icon, NULL);
-    xcb_property_set_handler(&globalconf.prophs, _NET_WM_PID, UINT_MAX,
-                             property_handle_net_wm_pid, NULL);
-    xcb_property_set_handler(&globalconf.prophs, _NET_WM_WINDOW_OPACITY, 1,
-                             property_handle_net_wm_opacity, NULL);
+    HANDLE_L(_NET_WM_NAME, property_handle_net_wm_name)
+    HANDLE_L(_NET_WM_ICON_NAME, property_handle_net_wm_icon_name)
+    HANDLE_L(_NET_WM_STRUT_PARTIAL, property_handle_net_wm_strut_partial)
+    HANDLE_L(_NET_WM_ICON, property_handle_net_wm_icon)
+    HANDLE_L(_NET_WM_PID, property_handle_net_wm_pid)
+    HANDLE_S(_NET_WM_WINDOW_OPACITY, property_handle_net_wm_opacity)
 
     /* background change */
-    xcb_property_set_handler(&globalconf.prophs, _XROOTPMAP_ID, 1,
-                             property_handle_xrootpmap_id, NULL);
+    HANDLE_S(_XROOTPMAP_ID, property_handle_xrootpmap_id)
+
+    /* If nothing was found, return */
+    END;
+
+#undef HANDLE_L
+#undef HANDLE_S
+#undef END
+
+    /* Get the property, if needed. */
+    xcb_get_property_reply_t *propr = NULL;
+    if(ev->state != XCB_PROPERTY_DELETE)
+    {
+        xcb_get_property_cookie_t cookie =
+            xcb_get_property(globalconf.connection, 0, ev->window, ev->atom,
+                             XCB_GET_PROPERTY_TYPE_ANY, 0, length);
+        propr = xcb_get_property_reply(globalconf.connection, cookie, 0);
+    }
+
+    (*handler)(ev->state, ev->window, ev->atom, propr);
+    p_delete(&propr);
 }
 
 // vim: filetype=c:expandtab:shiftwidth=4:tabstop=8:softtabstop=4:encoding=utf-8:textwidth=80
