@@ -58,7 +58,7 @@ wibox_widget_node_array_wipe(lua_State *L, int idx)
 {
     wibox_t *wibox = luaA_checkudata(L, idx, &wibox_class);
     foreach(widget_node, wibox->widgets)
-        luaA_object_unref_item(globalconf.L, idx, widget_node);
+        luaA_object_unref_item(globalconf.L, idx, widget_node->widget);
     widget_node_array_wipe(&wibox->widgets);
 }
 
@@ -417,6 +417,8 @@ wibox_systray_kickout(int phys_screen)
 static void
 wibox_systray_refresh(wibox_t *wibox)
 {
+    wibox->has_systray = false;
+
     if(!wibox->screen)
         return;
 
@@ -431,6 +433,8 @@ wibox_systray_refresh(wibox_t *wibox)
             xembed_window_t *em;
             int phys_screen = wibox->ctx.phys_screen;
 
+            wibox->has_systray = true;
+
             if(wibox->visible
                && systray->widget->isvisible
                && systray->geometry.width)
@@ -692,8 +696,13 @@ wibox_wipe(wibox_t *w)
 {
     if(w->window)
     {
+        int phys_screen = w->ctx.phys_screen;
+
         /* Activate BMA */
         client_ignore_enterleave_events();
+        /* Make sure we don't accidentally kill the systray window */
+        if(globalconf.screens.tab[phys_screen].systray.parent == w->window)
+            wibox_systray_kickout(phys_screen);
         xcb_destroy_window(globalconf.connection, w->window);
         /* Deactivate BMA */
         client_restore_enterleave_events();
@@ -1441,6 +1450,7 @@ luaA_wibox_set_widgets(lua_State *L, wibox_t *wibox)
     }
     /* duplicate table because next function will eat it */
     lua_pushvalue(L, -1);
+    luaA_object_unref_item(L, -4, wibox->widgets_table);
     wibox->widgets_table = luaA_object_ref_item(L, -4, -1);
     luaA_object_emit_signal(L, -3, "property::widgets", 0);
     wibox_need_update(wibox);
