@@ -151,7 +151,7 @@ window_opacity_get(xcb_window_t win)
 
     xcb_get_property_cookie_t prop_c =
         xcb_get_property_unchecked(globalconf.connection, false, win,
-                                   _NET_WM_WINDOW_OPACITY, CARDINAL, 0L, 1L);
+                                   _NET_WM_WINDOW_OPACITY, XCB_ATOM_CARDINAL, 0L, 1L);
 
     xcb_get_property_reply_t *prop_r =
         xcb_get_property_reply(globalconf.connection, prop_c, NULL);
@@ -207,7 +207,7 @@ window_takefocus(xcb_window_t win)
     ev.response_type = XCB_CLIENT_MESSAGE;
     ev.window = win;
     ev.format = 32;
-    ev.data.data32[1] = XCB_CURRENT_TIME;
+    ev.data.data32[1] = globalconf.timestamp;
     ev.type = WM_PROTOCOLS;
     ev.data.data32[0] = WM_TAKE_FOCUS;
 
